%%
%% Demo 1a: This demo evaluates different hyperspectral image classifiers
%%          We compare LDA, kNN, and SVM with and without spatial information 
%%  
%% Gustau Camps-Valls, gcamps@uv.es, http://isp.uv.es
%% 2014(c)
%%
%%  Exercises:
%%  
%%  1) Study the issue of low-sized training datasets, the ill-conditioning of covariance matrices in LDA, and the eventual use of PCA to alleviate this problem
%%  2) Compare the classification performance using hypespectral, spatial features and the combination under a stacked approach
%%  3) Try to improve the classification performance by ad hoc postprocessing rules
%%

clear;clc;close all;

addpath('svm_code/')

load('IndianPines.mat')

%% Spectral information
I = Xtotal;

%% Extract four spatial features from band=10
II = I(:,:,10);
se = strel('diamond',5);
t_hat = imtophat(II,se);
b_hat = imbothat(II,se);
se = strel('disk',5);
t_hat2  = imtophat(II,se);
b_hat2 = imbothat(II,se);
S = cat(3,t_hat, b_hat,t_hat2,b_hat2); % Spatial features cube

%% Labels
Y = Ytotal; % labels

%% Convert to data matrices
[rows cols bands] = size(I);
Xi = reshape(I,rows*cols,bands); % the 220 spectral bands are in columns
Xs = reshape(S,rows*cols,4);  % the four features are in columns

% X = [Xi,Xs];  % stack hyperspectral and spatial
% X = Xi;  % only hyperspectral
X = Xs;  % only spatial features

Y  = reshape(Y,rows*cols,1);

%% Setup the randomization seed
rand('seed',1234)
randn('seed',1234)

%% Split into training and test sets
C = max(unique(Y));
rateClass = 0.2;
Xtr = []; Ytr = [];
Xte = []; Yte = [];

% for c = 1:C % all classes
for c = [1 5 8 12] % only some classes of interest
    idc = find(Y==c);
    Xc = X(idc,:);
    Yc = Y(idc,:);
    samplesClass = length(idc);
    ntrain = round(rateClass*samplesClass);
    r = randperm(samplesClass);
    
    % Training pairs
    Xc_tr = Xc(r(1:ntrain),:);
    Yc_tr = Yc(r(1:ntrain),:);
    % Testing pairs
    Xc_te = Xc(r(ntrain+1:end),:);
    Yc_te = Yc(r(ntrain+1:end),:);
    
    % Store in train and test matrices
    Xtr = [Xtr; Xc_tr];
    Ytr = [Ytr; Yc_tr];
    Xte = [Xte; Xc_te];
    Yte = [Yte; Yc_te];
    
end

whos

%% Standardize data
Xtr = zscore(Xtr);
Xte = zscore(Xte);

%% Linear discriminant analysis
Ypred_lda = classify(Xte,Xtr,Ytr);
resLDA = assessment(Ypred_lda,Yte,'class')

%% k-NN
Ypred_knn = knnclassify(Xte,Xtr,Ytr);
resKNN = assessment(Ypred_knn,Yte,'class')

%% Support vector machine
Ypred_svm = svm_classify(Xte,Xtr,Ytr);
resSVM = assessment(Ypred_svm,Yte,'class')

%% Results

figure,bar([resLDA.OA, resKNN.OA, resSVM.OA])
set(gca,'XTickLabel',{'LDA','k-NN','SVM'})
title('Overall accuracy, OA[%]')
grid

figure,bar([resLDA.Kappa, resKNN.Kappa, resSVM.Kappa])
set(gca,'XTickLabel',{'LDA','k-NN','SVM'})
title('Kappa statistic, \kappa')
grid
