%%
%% Demo 2a: This demo shows how to extract standard spatial features from an image
%%  
%% Gustau Camps-Valls, gcamps@uv.es, http://isp.uv.es
%% 2014(c)
%%
%%  Exercises:
%%  
%%  1) Play around with different forms and extent of the structural elements 
%%  2) Code a function that takes a HSI, and returns a desired number of features in a 'spatial hypercube' ready for classification
%%

clear;clc;close all;

%% Matlab code to make nice figures
fontname = 'Times';
fontsize = 18;
fontunits = 'points';
set(0,'DefaultAxesFontName',fontname,'DefaultAxesFontSize',fontsize,'DefaultAxesFontUnits',fontunits,...
    'DefaultTextFontName',fontname,'DefaultTextFontSize',fontsize,'DefaultTextFontUnits',fontunits,...
    'DefaultLineLineWidth',3,'DefaultLineMarkerSize',8,'DefaultLineColor',[0 0 0]);

%% Auxiliary functions needed
addpath('aux_code/')

%% Load image
load IndianPines.mat
I = Xtotal;

%% RGB composition
figure,imagesc(I(:,:,[40 30 10])/max(I(:))),axis off square
print -depsc figures/rgb.eps

%% Select just one band for analysis:
II = squeeze(I(:,:,3));

%% Markov random fields
M = nlfilter(II,[3 3],@mrf);
figure, imagesc(M),axis off square,title('Entropy of the Markov random field')
print -depsc figures/markov.eps

%% Erosion
% Note: you can modify the profile to use other structural elements...
se = strel('disk',3);
m = imerode(II,se);
figure,imagesc(m), title('Erosion, disk 3x3'),axis off square
print -depsc figures/erode.eps

%% Dilation
se = strel('disk',3);
m = imdilate(II,se);
figure,imagesc(m), title('Dilation, disk 3x3'),axis off square
print -depsc figures/dilate.eps

%% Opening
se = strel('disk',3);
m = imopen(II,se);
figure,imagesc(m), title('Opening, disk 3x3'),axis off square
print -depsc figures/opening.eps

%% Tophat/Bottomhat
se = strel('diamond',5);
t_hat = imtophat(II,se);
b_hat = imbothat(II,se);
se = strel('disk',5);
t_hat2  = imtophat(II,se);
b_hat2 = imbothat(II,se);
figure,imagesc(t_hat), title('Top hat, diamond 3x3'),axis off square
print -depsc figures/tophat.eps
figure,imagesc(b_hat), title('Bottom hat, disk 3x3'),axis off square
print -depsc figures/bottomhat.eps

%% Morphological profile
depth = 3;
MP    = morphological_profile(II,depth,0);
for i=1:2*depth+1
    figure,imagesc(squeeze(MP(:,:,i))), 
    if i<=depth
        title(['MP, Close ' num2str(i)]),axis off square
    elseif i>depth+1
        title(['MP, Open ' num2str(i)]),axis off square
    elseif i==(depth+1)
        title(['Image']),axis off square
    end
    eval(['print -depsc figures/mp' num2str(i) '.eps'])
end

%% Edges
EDGES1 = double(edge(II,'canny'));
EDGES2 = double(edge(II,'prewitt'));
figure,imagesc(EDGES1), title('Canny edges'),axis off square
print -depsc figures/canny.eps
figure,imagesc(EDGES2), title('Prewitt edges'),axis off square
print -depsc figures/prewitt.eps

%% Standard deviation
S = stdfilt(II);
figure,imagesc(S),title('Local standard deviation, 3x3 window'),axis off square
print -depsc figures/stdev.eps

%% Range filter
R = rangefilt(II,ones(5));
figure,imagesc(R),title('Local range filter, 5x5 window'),axis off square
print -depsc figures/rangefilter.eps

%% Entropy
H = entropyfilt(II/max(II(:)));
figure,imagesc(H),title('Local entropy, 9x9 window'),axis off square
print -depsc figures/entropy.eps

%% Curvature
fx = imfilter(II,fspecial('Sobel'),'conv','replicate');
fy = imfilter(II,fspecial('Sobel')','conv','replicate');
modGrad = (fx.*fx + fy.*fy).^.2;
fx = fx./(modGrad+eps);
fy = fy./(modGrad+eps);
fyy = imfilter(fy,fspecial('Sobel')','conv','replicate');
fxx = imfilter(fx,fspecial('Sobel')','conv','replicate');
curvatura=fyy+fxx;
figure,imagesc(curvatura),title('Curvature/derivative'),axis off square
print -depsc figures/curvature.eps

%% Median filter
MED = medfilt2(II);
figure,imagesc(MED),title('Median filter, 3x3 window'),axis off square
print -depsc figures/median.eps

%% Mean filter
H = ones(3);
MF = imfilter(II, H);
figure,imagesc(MF),title('Mean filter, 3x3 window'),axis off square
print -depsc figures/mean.eps

%% Max pooling filtering
maxpool = ordfilt2(II,9,true(3));
figure,imagesc(maxpool),title('Max pooling, 3x3 window'),axis off square
print -depsc figures/maxpooling.eps

%% Wavelet decomposition
WT = wavelet_decomposition(II,1); % flag=1 for printing the figures

% %% Watch out! Spectral features, not spatial! PCA components
% X = reshape(Xtotal,145*145,220);
% C = cov(X);
% [V, D] = eigs(C,220);
% figure,semilogy(diag(D)/max(D(:))),grid, axis square,title('(Normalized) Eigenvalues')
% Xp = X*V(:,1:9);
% Ip = reshape(Xp,145,145,9);
% for pc=1:9
%     figure,imagesc(squeeze(Ip(:,:,pc))), axis off square,title(['PC ' num2str(pc)])
%     eval(['print -depsc figures/PC' num2str(pc) '.eps'])
% end


