%%
%% Demo 2b: This demo shows the performance of linear and kernel supervised 
%%          and unsupervised (spectral) feature extraction for hyperspectral image classification
%
%% Gustau Camps-Valls, gcamps@uv.es, http://isp.uv.es
%% 2014(c)
%%
%%  Exercises:
%   1) Study the impact of the kernel lengthscale (sigma value)
%   2) Is there an optimal sigma for both KPCA and KOPLS?
%   3) Discuss the main advantages and problems of the kernel approaches to feature extraction

%% Clean the workspace, close all figures, and remove variables from memory
clear;clc;close all;
rand('seed',1234)
randn('seed',1234)

%% Auxiliary functions needed
addpath('aux_code')

%% DATA

load('IndianPines.mat')

% Size of the hypercube: rows x columns x bands
[r c b] = size(Xtotal);

% Number of classes: 0 (background), 1, ..., 16
Ytotal = Ytotal(:);
NumClasses = max(Ytotal)

% Image to Matrix
XX = reshape(Xtotal,r*c,b);

% Labels to column
YY = Ytotal(:);

% Important: avoid background data for training/testing
nozero = find(Ytotal~=0);
X = XX(nozero,:);
Y = YY(nozero,:);

% Split training and validation set
[n d] = size(X);
ran = randperm(n);
Xtrain = X(ran(1:500),:);
Ytrain = Y(ran(1:500),:);
Xvalid = X(ran(501:2000),:);
Yvalid = Y(ran(501:2000),:);
[ntr d] = size(Xtrain);

%% Binarize labels (without the background class)
Ytrainb = binarize(Ytrain);

%% Feature extraction

% PCA
C = cov(Xtrain);
Nf = rank(C);
[V_PCA D_PCA] = eigs(C,Nf);

% KPCA
sigma = mean(pdist(Xvalid)) / 10;  % heuristic
K = kernelmatrix('rbf',Xtrain',Xtrain',sigma); % n x n
Ktr = kernelcentering(K);
Kva = kernelmatrix('rbf',Xvalid',Xtrain',sigma); % m x n
Kva = kernelcentering(Kva',Ktr)';
Nf = rank(Ktr);
[V_KPCA D_KPCA] = eigs(Ktr,Nf);

% OPLS
Nf = rank(Xtrain'*Ytrainb);
[V_OPLS,D_OPLS] = gen_eig((Xtrain'*Ytrainb)*(Ytrainb'*Xtrain),Xtrain'*Xtrain,Nf);

% KOPLS
Ky = Ytrainb*Ytrainb';
Ky = kernelcentering(Ky);
aux1 = Ktr*Ky*Ktr;
aux2 = Ktr'*Ktr;
Nf = rank(Ktr*Ytrainb);
[V_KOPLS d] = gen_eig(aux1,aux2,Nf);

% Maximum number of features that can be extracted
Nfmax = [rank(C), rank(Xtrain'*Ytrainb), rank(Ktr), rank(Ktr*Ytrainb)]-1

% Classification using a restricted number of features/components
results = NaN(4,max(Nfmax)); % initialize results matrix
for ind_meth = 1:4
    for ind_feat = 1:min(300,Nfmax(ind_meth))
        
        % Dimensionality reduction
        if ind_meth == 1
            % PCA
            auxXtrain = Xtrain*V_PCA(:,1:ind_feat);
            auxXvalid = Xvalid*V_PCA(:,1:ind_feat);
        elseif ind_meth == 2
            % OPLS
            auxXtrain = Xtrain*V_OPLS(:,1:ind_feat);
            auxXvalid = Xvalid*V_OPLS(:,1:ind_feat);
        elseif ind_meth == 3
            % KPCA
            auxXtrain = Ktr*V_KPCA(:,1:ind_feat);
            auxXvalid = Kva*V_KPCA(:,1:ind_feat);
        elseif ind_meth == 4
            % KOPLS
            auxXtrain = Ktr*V_KOPLS(:,1:ind_feat);
            auxXvalid = Kva*V_KOPLS(:,1:ind_feat);
        end
        
        % Winner-takes-all classifier
        w  = pinv(auxXtrain)*Ytrainb;
        Yp = auxXvalid*w;
        [val,Ypred] = max(Yp,[],2);

        % Compute results
        r = assessment(Yvalid,Ypred,'class');
        results(ind_meth,ind_feat) =  r.Kappa;
        
    end
end

% Plot results
figure
semilogx(results(1,:),'r','linewidth',3)
hold on,
semilogx(results(2,:),'k','linewidth',3)
semilogx(results(3,:),'r','linewidth',5)
semilogx(results(4,:),'k','linewidth',5)
xlabel('#Features')
ylabel('Kappa')
legend('PCA','OPLS','KPCA','KOPLS','Location','SouthEast')
grid
