%%
%% Demo 3c: This demo evaluates different techniques for abundance estimation
%%
%%          In particular, we compare classical and advanced methods, belonging to different families: VCA, PPI, NFINDR, SVDD
%%
%%          Several figures are generated:
%%              - Abundance maps using an unconstrained least squares linear regression
%%              - Plot the extracted endmembers versus the closest (in MSE terms) material in the USGS database
%%              - Bar plots of CPU time, relative RMSE, SAM and SID
%%
%%          Main parts of the code belong to the following online toolboxes:
%%              Jose Bioucas-Dias, http://www.lx.it.pt/~bioucas/code.htm
%%              Manuel Grana & Miguel Angel Veganzones, http://www.ehu.es/computationalintelligence
%%
%%          Please acknowledge their usage if used in your work!
%%
%% Gustau Camps-Valls, gcamps@uv.es, http://isp.uv.es
%% 2014(c)
%%

clear;clc;close all;
fontname = 'Arial';
fontsize = 18;
fontunits = 'points';
set(0,'DefaultAxesFontName',fontname,'DefaultAxesFontSize',fontsize,'DefaultAxesFontUnits',fontunits,...
    'DefaultTextFontName',fontname,'DefaultTextFontSize',fontsize,'DefaultTextFontUnits',fontunits,...
    'DefaultLineLineWidth',1.5,'DefaultLineMarkerSize',8,'DefaultLineColor',[0 0 0]);
rand('seed',1234)
randn('seed',1234)

%% Add some needed funtions to the path
addpath('aux_code/')

%% Load results obtained with demo3b.m
load results_endmembers.mat

%% Load true signatures
load USGS_1995_Library

%% Plot estimated signatures per model, and their abundances
verbose=0

%% Plot endmembers identified and the closest material in the USGS database
%% Plot also the abundance maps for all extraction methods using a LS approach
for k=1:length(methods)
    ['endmembers_' methods{k}]
    endmembers = eval(['endmembers_' methods{k}]);
    
    if k==9
        endmembers=endmembers';
    end
    
    % Search the closest correspondent endmembers in USGS db
    [ind names] = searchUSGS(endmembers,BANDS)
    % This spectra constitute the ground truth
    Mtrue = datalib(BANDS,ind);
    % Sort the found endmembers to match the closest ones in the USGS db
    endmembers = correspondences(Mtrue,endmembers);
    % Multiply the found endmember by a factor such that minimizes the RMSE
    % (just for visualization purposes)
    endmembers = adjustScale(Mtrue,endmembers);
    
    % LS abundance estimation (no constraints!)
    s = (pinv(endmembers) * x)';
    S = reshape(s,[Lines Columns p]);
    
    E = NaN*ones(size(wavlen),p);
    
    E(BANDS,:) = endmembers;
    T = NaN*ones(size(wavlen),p);
    T(BANDS,:) = Mtrue;
    
    n=ceil(sqrt(p));
    
    if verbose
    % Plot the estimated and the true signatures
    n=ceil(sqrt(p));
    for i= 1:p
        imagesc(S(:,:,i));
        colormap cmr%gray
        axis('off','equal')
        title(['USGS #' int2str(ind(i))])
        eval(['print -depsc2 figures/LSmap_' methods{k} '_endmember_' num2str(i)])
        
        pause
        
        plot(wavlen,E(:,i),'b-',wavlen,T(:,i),'r-','Linewidth',2);
        axis([400 2500 0 1])
        grid
        title(['USGS #' int2str(ind(i))])
        xlabel('\lambda [\mum]'); ylabel('Norm. Reflectance');
        eval(['print -depsc2 figures/signature_' methods{k} '_endmember_' num2str(i)])
        
        pause
    end
    end
    % The CPU cost
    TIME(k) = eval(['cputime_' methods{k}]);
    
    %  The distances to the true (measured) signals, i.e. the signatures
    % Compute the relative RMSE
    ERROR(k) = norm(Mtrue-endmembers,'fro')/norm(Mtrue,'fro');
    % Compute SAM and SID
    [SAM(k) SID(k)] = errors(Mtrue,endmembers,length(BANDS));
    
end

% close all
figure(1),
bar(1:length(TIME),TIME,'k'),
set(gca,'XTickLabel',methods_lab,'fontsize',20);
xlabel('Methods'),
ylabel('CPU time [s]'),
grid
print -depsc figures/cpucost.eps

figure(2),
bar(1:length(ERROR),ERROR,'k'),
set(gca,'XTickLabel',methods_lab,'fontsize',20);
xlabel('Methods'),
ylabel('Relative RMSE'),
grid
print -depsc figures/RMSE.eps

figure(3),
bar(1:length(SAM),SAM,'k'),
set(gca,'XTickLabel',methods_lab,'fontsize',20);
xlabel('Methods'),
ylabel('SAM, \theta [degrees]'),
grid
print -depsc figures/SAM.eps

figure(4),
bar(1:length(SID),SID,'k'),
set(gca,'XTickLabel',methods_lab,'fontsize',20);
xlabel('Methods'),
ylabel('SID [degrees]'),
grid
print -depsc figures/SID.eps

tile

