%%
%% Demo 4: This demo evaluates the fusion of HSI and LiDAR for image classification 
%%         We compare LDA, kNN, and SVM with and without fusion 
%%  
%% Gustau Camps-Valls, gcamps@uv.es, http://isp.uv.es
%% 2014(c)
%% ---- See copyright.txt file for the description of copyright issues related to the use of the data
%% ---- The full dataset can be obtained from http://hyperspectral.ee.uh.edu/?page_id=459
%%
%%  Exercises:
%%  
%%  1) Study the issue of low-sized training datasets, the ill-conditioning of covariance matrices in LDA, and the eventual use of PCA to alleviate this problem
%%  2) Compare the classification performance using hypespectral, LiDAR and the combination under a stacked approach
%%  3) Try to improve the classification performance by concatenation of spatial/contextual features
%%  4) Try to improve the classification performance by ad hoc postprocessing rules
%%

clear;clc;close all;

% These files contain the original full data, available at http://hyperspectral.ee.uh.edu/?page_id=459
% I = double(imread('2013_IEEE_GRSS_DF_Contest_CASI.tif'));
% L = double(imread('2013_IEEE_GRSS_DF_Contest_LiDAR.tif'));
% load classLabels.mat

load classLabels_chip.mat 
load 2013_IEEE_GRSS_DF_Contest_CASI_chip.mat I2 
load 2013_IEEE_GRSS_DF_Contest_LiDAR_chip.mat L2
I=I2;L=L2;Y=Y2;
addpath('./svm_code/')

%% Convert to data matrices
[rows cols bands] = size(I);
Xi = reshape(I,rows*cols,bands);
Xl = reshape(L,rows*cols,1);

% X = [Xi,Xl];  % stack hyperspectral and LiDAR
% X = Xi;  % only hyperspectral
X = Xl;  % only LiDAR

Y  = reshape(Y,rows*cols,1);

%% Setup the randomization seed
rand('seed',1234)
randn('seed',1234)

%% Split into training and test sets
C = max(unique(Y));
rateClass = 0.2;
Xtr = []; Ytr = [];
Xte = []; Yte = [];

for c = 1:C
    idc = find(Y==c);
    Xc = X(idc,:);
    Yc = Y(idc,:);
    samplesClass = length(idc);
    ntrain = round(rateClass*samplesClass);
    r = randperm(samplesClass);
    
    % Training pairs
    Xc_tr = Xc(r(1:ntrain),:);
    Yc_tr = Yc(r(1:ntrain),:);
    % Testing pairs
    Xc_te = Xc(r(ntrain+1:end),:);
    Yc_te = Yc(r(ntrain+1:end),:);
    
    % Store in train and test matrices
    Xtr = [Xtr; Xc_tr];
    Ytr = [Ytr; Yc_tr];
    Xte = [Xte; Xc_te];
    Yte = [Yte; Yc_te];
    
end

whos

%% Standardize data
Xtr = zscore(Xtr);
Xte = zscore(Xte);
X = zscore(X);

%% Linear discriminant analysis
Ypred_lda = classify(Xte,Xtr,Ytr);
Ypred_lda_all = classify(X,Xtr,Ytr);
resLDA = assessment(Ypred_lda,Yte,'class')

%% k-NN
Ypred_knn = knnclassify(Xte,Xtr,Ytr);
Ypred_knn_all = knnclassify(X,Xtr,Ytr);
resKNN = assessment(Ypred_knn,Yte,'class')

%% Support vector machine
Ypred_svm = svm_classify(Xte,Xtr,Ytr);
Ypred_svm_all = svm_classify(X,Xtr,Ytr);
resSVM = assessment(Ypred_svm,Yte,'class')

%% Results

figure,bar([resLDA.OA, resKNN.OA, resSVM.OA])
set(gca,'XTickLabel',{'LDA','k-NN','SVM'})
title('Overall accuracy, OA[%]')
grid

figure,bar([resLDA.Kappa, resKNN.Kappa, resSVM.Kappa])
set(gca,'XTickLabel',{'LDA','k-NN','SVM'})
title('Kappa statistic, \kappa')
grid

%% Classification maps
GT = reshape(Y,rows,cols);
RGB = reshape(I(:,:,[40 30 10]),[rows,cols,3]);
Y1 = reshape(Ypred_lda_all,rows,cols);
Y3 = reshape(Ypred_knn_all,rows,cols);
Y4 = reshape(Ypred_svm_all,rows,cols);
figure,imagesc(GT),axis off square, title('Ground truth')
figure,imagesc(RGB/max(RGB(:))),axis off square, title('RGB')
figure,imagesc(Y1),axis off square, title('LDA classification map')
figure,imagesc(Y3),axis off square, title('k-NN classification map')
figure,imagesc(Y4),axis off square, title('SVM classification map')

