
%%
%% Demo 5: Linear regression and kernel ridge regression (KRR) to estimate 
%%         biophysical parameters from CHRIS hyperspectral data
%%
%% Gustau Camps-Valls, gcamps@uv.es, http://isp.uv.es
%% 2014(c)
%%
%%
%% Exercise: 
%%     1) Code a LR and a KRR to predict the three variables simultaneously
%%           ---Hint: note that computing either the primal or dual weights is independent of the output dimensionality!
%%

%% Clean the workspace, close all figures, and remove variables from memory
clear;clc;close all;
warning off

%% Add some needed funtions to the path
addpath('aux_code/')

%% DATA
load SPARC0304.mat
load CHRIS030712snippet.mat

% Rename the input-output variables
X = Spectra';  % 135 samples/points in 62-dimensional space (spectral channels of CHRIS sensor)
Y = [Chla',LAI',fcover'];  % Three biophysical parameters to be predicted (independently or jointly!)
Xtest = reshape(I,[200*200,62]);

% Size
[samples dims] = size(X);

% Band selection since there are some low SNR bands in the acquired image 
bad = [1 2 3 21:27];
good = setdiff(1:62,bad);

% Select training and test data
rand('seed',1)
r = randperm(samples);

% Training data: randomly select Ntr pixels
Ntr = 90
Xtrain   = X(r(1:Ntr),good);
Ytrain   = Y(r(1:Ntr),1);      % If we just want to predict chlorophyll content

% Validation data: the remaining pixels
Xvalid   = X(r(Ntr+1:end),good);
Yvalid   = Y(r(Ntr+1:end),1);    % If we just want to predict chlorophyll content

% Test data: the whole image snippet of 200x200 pixels
Xtest    = Xtest(:,good); 

%% Scale data
Xtrain = scale(Xtrain);
Xvalid = scale(Xvalid);
Xtest  = scale(Xtest);


%% Regularized least squares linear regression (KRR using the linear kernel)
LAMBDAS = logspace(-10,10,10); % try 10 different values of the regularization parameter
Ktrain = Xtrain*Xtrain';       % n x n
Kvalid = Xvalid*Xtrain';       % m x n
Ktest  = Xtest*Xtrain';       % m x n

% Lambda fitting by standard cross-validation
res = [];
i=0;
for lambda = LAMBDAS
    i=i+1;
    % Dual weights
    alpha = inv(Ktrain + lambda*eye(size(Ktrain,1)))*Ytrain;  
    Ypred = Kvalid*alpha;  % (m x n)*(n x 1) = m x 1
    % Compute results for all the data points
    results = assessment(Yvalid,Ypred,'regress');
    res_LR(i,:) = [lambda results.RMSE];
end

% Find the best 'lambda' parameter
[RMSE_LR idx] = min(res_LR(:,2));
lambda_LR   = res_LR(idx,1);

% Fit the weights again using the best regularization parameter
alpha_LR = inv(Ktrain + lambda_LR*eye(size(Ktrain,1)))*Ytrain; 
% Compute predictions for the validation set
Ypred_LR = Kvalid*alpha_LR;
% Compute the chlorophyll content map (the final test data)
Ypred_test_LR = Ktest*alpha_LR;

%% Nonlinear (kernel-based) regression
%   Kernel Ridge Regression
%   aka Least squares support vector machine

% Sigma and Lambda fitting by xvalidation
LAMBDAS = logspace(-10,10,20);
SIGMAS  = logspace(-3,3,20);

i=0;
res = [];
for sigma = SIGMAS
    Ktrain = kernelmatrix('rbf',Xtrain',Xtrain',sigma); % n x n
    Kvalid = kernelmatrix('rbf',Xvalid',Xtrain',sigma); % m x n
    
    for lambda = LAMBDAS
        i=i+1;
        alpha = inv(Ktrain + lambda*eye(size(Ktrain,1)))*Ytrain;  
        Ypred = Kvalid*alpha;  % (m x n)*(n x 1) = m x 1
        
        % Compute results for all the data points
        results = assessment(Yvalid,Ypred,'regress');        
        res_KRR(i,:) = [sigma lambda results.RMSE];
    end
end

clc
% Find the best pair of (sigma, lambda) values
[RMSE_KRR idx] = min(res_KRR(:,3));
sigma  = res_KRR(idx,1);
lambda = res_KRR(idx,2);

% Test 
Ktrain = kernelmatrix('rbf',Xtrain',Xtrain',sigma);
Kvalid = kernelmatrix('rbf',Xvalid',Xtrain',sigma);
Ktest  = kernelmatrix('rbf',Xtest',Xtrain',sigma);
alpha = inv(Ktrain + lambda*eye(size(Ktrain,1)))*Ytrain;  
Ypred_KRR = Kvalid*alpha;
Ypred_test_KRR = Ktest*alpha;

%% Plot results

figure,
imagesc(reshape(Ypred_test_LR,200,200))
axis square off,
title(['Chlorophyll prediction with LR, RMSE: ' num2str(RMSE_LR)] )
colorbar

figure,
imagesc(reshape(Ypred_test_KRR,200,200))
axis square off,
title(['Chlorophyll prediction with KRR, RMSE: ' num2str(RMSE_KRR)] )
colorbar

figure,
scatterhist(Yvalid(:,1),Ypred_LR(:,1)),
xlabel('Observation')
ylabel('Prediction')
title(['Chlorophyll prediction with LR, RMSE: ' num2str(RMSE_LR)] )
grid on

figure,
scatterhist(Yvalid(:,1),Ypred_KRR(:,1)),
xlabel('Observation')
ylabel('Prediction')
title(['Chlorophyll prediction with KRR, RMSE: ' num2str(RMSE_KRR)] )
grid on

tile


